# -*- coding: utf-8 -*-


class Emoji:
    def __init__(self, data=None):
        self.id: str = ""
        self.type: int = 0
        if data:
            self.__dict__ = data


class EmojiType:
    system = 1
    emoji = 2



# -*- coding: utf-8 -*-


class ChannelType:
    TEXT_CHANNEL = 0  # 文字子频道
    # x_CHANNEL = 1    # 保留，不可用
    VOICE_CHANNEL = 2  # 语音子频道
    # x_CHANNEL = 3    # 保留，不可用
    GROUP_CHANNEL = 4  # 子频道分组
    LIVE_CHANNEL = 10005  # 直播子频道
    APP_CHANNEL = 10006  # 应用子频道
    DISCUSSION_CHANNEL = 10007  # 论坛子频道


class ChannelSubType:
    TALK = 0  # 闲聊
    POST = 1  # 公告
    CHEAT = 2  # 攻略
    BLACK = 3  # 开黑


class PrivateType:
    PUBLIC = 0  # 公开频道
    ADMIN = 1  # 管理员和群主可见
    SPECIFIED_USER = 2  # 群主管理员+指定成员，可使用 修改子频道权限接口 指定成员


class SpeakPermission:
    INVALID = 0  # 无效类型
    EVERYONE = 1  # 所有人
    ADMIN = 2  # 群主管理员+指定成员，可使用 修改子频道权限接口 指定成员


class Channel:
    def __init__(self, data=None):
        self.id: str = ""
        self.guild_id: str = ""
        self.name: str = ""
        self.type: int = 0
        self.sub_type: int = 0
        self.position: int = 0
        self.parent_id: str = ""
        self.owner_id: str = ""
        self.private_type: int = 0
        self.speak_permission: int = 0
        self.application_id: str = ""
        self.permissions: str = ""
        if data is not None:
            self.__dict__ = data


class CreateChannelRequest:
    def __init__(
        self,
        name: str,
        channel_type: ChannelType,
        sub_type: ChannelSubType,
        position: int = 0,
        parent_id: str = "",
    ):
        self.parent_id = parent_id
        self.position = position
        self.sub_type = sub_type
        self.name = name
        self.type = channel_type


class PatchChannelRequest:
    def __init__(
        self, name: str, channel_type: ChannelType, position: int, parent_id: str
    ):
        self.parent_id = parent_id
        self.channel_type = channel_type
        self.position = position
        self.name = name


class ChannelResponse:
    def __init__(self, data=None):
        self.id: str = ""
        self.guild_id: str = ""
        self.name: str = ""
        self.type: int = 0
        self.sub_type: int = 0
        self.position: int = 0
        self.parent_id: str = ""
        self.owner_id: str = ""
        if data is not None:
            self.__dict__ = data



from typing import List


class Permission:
    def __init__(self, type: int, specify_role_ids: List[str] = None, specify_user_ids: List[str] = None):
        if type:
            self.type = type
        if specify_role_ids:
            self.specify_role_ids = specify_role_ids
        if specify_user_ids:
            self.specify_user_ids = specify_user_ids


class RenderData:
    def __init__(self, label: str, visited_label: str, style: int):
        self.label = label
        self.visited_label = visited_label
        self.style = style


class Action:
    def __init__(self, type: int, permission: Permission, click_limit: int, data: str, at_bot_show_channel_list: bool):
        self.type = type
        self.permission = permission
        self.click_limit = click_limit
        self.data = data
        self.at_bot_show_channel_list = at_bot_show_channel_list


class Button:
    def __init__(self, id: str, render_data: RenderData, action: Action):
        self.id = id
        self.render_data = render_data
        self.action = action


class InlineKeyboardRow:
    def __init__(self, buttons: List[Button]):
        self.buttons = buttons


class InlineKeyboard:
    def __init__(self, rows: List[InlineKeyboardRow]):
        self.rows = rows





class WsContext:
    """
    被动事件里携带的上下文信息，目前仅有部分事件支持
    """

    def __init__(self, event_type: str, event_id: str):
        self.event_type = str(event_type or "")
        self.event_id = str(event_id or "")



# -*- coding: utf-8 -*-
from typing import List

from qqbot.model.inline_keyboard import InlineKeyboard
from qqbot.model.member import User, Member


class MessageGet:
    def __init__(self, data=None):
        self.message: Message = Message()
        if data:
            self.__dict__ = data


class Message:
    def __init__(self, data=None):
        self.id: str = ""
        self.channel_id: str = ""
        self.guild_id: str = ""
        self.content: str = ""
        self.timestamp: str = ""
        self.edited_timestamp: str = ""
        self.mention_everyone: str = ""
        self.author: User = User()
        self.attachments: List[MessageAttachment] = [MessageAttachment()]
        self.embeds: List[MessageEmbed] = [MessageEmbed()]
        self.mentions: List[User] = [User()]
        self.member: Member = Member()
        self.ark: MessageArk = MessageArk()
        self.seq: int = 0
        self.seq_in_channel = ""
        self.message_reference = MessageReference()
        self.src_guild_id = ""
        if data:
            self.__dict__ = data


class TypesEnum:
    around = "around"
    before = "before"
    after = "after"
    latest = ""


class MessagesPager:
    def __init__(self, type: TypesEnum, id: str, limit: str):
        self.type = type
        self.id = id
        self.limit = limit


class MessageAttachment:
    def __init__(self, url: str = "", data=None):
        self.url = url
        if data:
            self.__dict__ = data


class MessageEmbedThumbnail:
    def __init__(self, url: str = "", data=None):
        # 图片地址
        self.url = url
        if data is not None:
            self.__dict__ = data


class MessageEmbedField:
    def __init__(self, data=None, name: str = None, value: str = None):
        self.name = name
        self.value = value
        if data:
            self.__dict__ = data


class MessageEmbed:
    def __init__(
        self,
        title: str = "",
        prompt: str = "",
        thumbnail: MessageEmbedThumbnail = MessageEmbedThumbnail(),
        fields: List[MessageEmbedField] = [MessageEmbedField()],
        data=None,
    ):
        # 标题
        self.title = title
        # 消息弹窗内容
        self.prompt = prompt
        # 缩略图
        self.thumbnail = thumbnail
        # 消息创建时间
        self.fields = fields
        if data:
            self.__dict__ = data


class MessageArkObjKv:
    def __init__(self, data=None, key: str = None, value: str = None):
        self.key = key
        self.value = value
        if data:
            self.__dict__ = data


class MessageArkObj:
    def __init__(self, data=None, obj_kv: List[MessageArkObjKv] = None):
        self.obj_kv = obj_kv
        if data:
            self.__dict__ = data


class MessageArkKv:
    def __init__(
        self,
        data=None,
        key: str = None,
        value: str = None,
        obj: List[MessageArkObj] = None,
    ):
        self.key = key
        self.value = value
        self.obj = obj
        if data:
            self.__dict__ = data


class MessageArk:
    def __init__(
        self,
        template_id: int = 0,
        kv: List[MessageArkKv] = [MessageArkKv()],
        data=None,
    ):
        self.template_id = template_id
        self.kv = kv
        if data:
            self.__dict__ = data


class MessageMarkdownParams:
    def __init__(self, data=None, key="", values: List[str] = None):
        self.key = key
        self.values = values
        if data:
            self.__dict__ = data


class MessageMarkdown:
    def __init__(self, data=None, template_id: int = 0, params=None, content: str = ""):
        self.template_id = template_id
        self.params = params
        self.content = content
        if data:
            self.__dict__ = data


class MessageReference:
    def __init__(self, data=None, message_id: str = "", ignore_get_message_error: bool = False):
        self.message_id = message_id
        self.ignore_get_message_error = ignore_get_message_error
        if data:
            self.__dict__ = data


class MessageKeyboard:
    """
    id 和 content 两个参数互斥，都传值将返回错误
    """
    def __init__(self, id: str = None, content: InlineKeyboard = None):
        if id:
            self.id = id
        if content:
            self.content = content


class MessageSendRequest:
    def __init__(
        self,
        content: str = "",
        msg_id: str = None,
        embed: MessageEmbed = None,
        ark: MessageArk = None,
        image: str = "",
        message_reference: MessageReference = None,
        markdown: MessageMarkdown = None,
        keyboard: MessageKeyboard = None
    ):
        """
        机器人发送消息时所传的数据对象

        :param content: 消息内容，文本内容，支持内嵌格式
        :param msg_id: 要回复的消息id(Message.id), 在 AT_CREATE_MESSAGE 事件中获取。带了 msg_id 视为被动回复消息，否则视为主动推送消息
        :param embed: embed 消息，一种特殊的 ark
        :param ark: ark 消息
        :param image: 图片url地址
        :param message_reference: 引用消息
        :param markdown: markdown 消息
        :param keyboard: markdown 消息内的按钮
        """

        self.content = content
        self.embed = embed
        self.ark = ark
        self.image = image
        self.msg_id = msg_id
        self.message_reference = message_reference
        self.markdown = markdown
        self.keyboard = keyboard


class DirectMessageGuild:
    def __init__(self, data=None, guild_id: str = "", channel_id: str = "", creat_time: str = ""):
        self.guild_id =  guild_id
        self.channel_id =  channel_id
        self.creat_time =  creat_time
        if data is not None:
            self.__dict__ = data


class CreateDirectMessageRequest:
    def __init__(self, source_guild_id: str, user_id: str):
        """
        机器人发送私信时所传的数据对象

        :param source_guild_id: 创建的私信频道ID
        :param user_id: 私信接收人用户ID
        """
        self.recipient_id = user_id
        self.source_guild_id = source_guild_id


class DeletedMessageOriginalAuthor:
    def __init__(self, data=None):
        self.id = ""
        self.username = ""
        self.bot = False
        if data:
            self.__dict__ = data


class DeletedMessage:
    def __init__(self, data=None):
        self.guild_id = ""
        self.src_guild_id = ""
        self.channel_id = ""
        self.id = ""
        self.author = DeletedMessageOriginalAuthor()
        if data:
            self.__dict__ = data


class DeletionOperator:
    def __init__(self, data=None):
        self.id = ""
        if data:
            self.__dict__ = data


class DeletedMessageInfo:
    def __init__(self, data=None):
        self.message = DeletedMessage()
        self.op_user = DeletionOperator()
        if data:
            self.__dict__ = data



# -*- coding: utf-8 -*-
from qqbot.model.member import Member


class Schedule:
    def __init__(
        self,
        data=None,
        id: str = None,
        name: str = None,
        description: str = None,
        start_timestamp: str = None,
        end_timestamp: str = None,
        creator: Member = None,
        jump_channel_id: str = None,
        remind_type: str = None,
    ):
        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        if description is not None:
            self.description = description
        if start_timestamp is not None:
            self.start_timestamp = start_timestamp
        if end_timestamp is not None:
            self.end_timestamp = end_timestamp
        if creator is not None:
            self.creator = creator
        if jump_channel_id is not None:
            self.jump_channel_id = jump_channel_id
        if remind_type is not None:
            self.remind_type = remind_type
        if data is not None:
            self.__dict__ = data


class GetSchedulesRequest:
    def __init__(self, since: int):
        self.since = since


class ScheduleToCreate:
    def __init__(
        self,
        name: str,
        start_timestamp: str,
        end_timestamp: str,
        remind_type: str,
        description: str = None,
        creator: Member = None,
        jump_channel_id: str = None,
    ):
        self.schedule = Schedule(
            name=name,
            start_timestamp=start_timestamp,
            end_timestamp=end_timestamp,
            remind_type=remind_type,
            description=description,
            creator=creator,
            jump_channel_id=jump_channel_id,
        )


class ScheduleToPatch:
    def __init__(
        self,
        start_timestamp: str,
        end_timestamp: str,
        remind_type: str,
        name: str = None,
        description: str = None,
        creator: Member = None,
        jump_channel_id: str = None,
    ):
        self.schedule = Schedule(
            name=name,
            start_timestamp=start_timestamp,
            end_timestamp=end_timestamp,
            remind_type=remind_type,
            description=description,
            creator=creator,
            jump_channel_id=jump_channel_id,
        )



# -*- coding: utf-8 -*-

from enum import Enum


class Intents(Enum):
    INTENT_GUILDS = 1 << 0
    # - GUILD_CREATE // 当机器人加入新guild时
    # - GUILD_UPDATE // 当guild资料发生变更时
    # - GUILD_DELETE // 当机器人退出guild时
    # - CHANNEL_CREATE // 当channel被创建时
    # - CHANNEL_UPDATE // 当channel被更新时
    # - CHANNEL_DELETE // 当channel被删除时

    INTENT_GUILD_MEMBERS = 1 << 1
    # - GUILD_MEMBER_ADD // 当成员加入时
    # - GUILD_MEMBER_UPDATE // 当成员资料变更时
    # - GUILD_MEMBER_REMOVE // 当成员被移除时

    INTENT_GUILD_MESSAGES = 1 << 9
    # - MESSAGE_CREATE         // 发送消息事件，代表频道内的全部消息，而不只是 at 机器人的消息。内容与 AT_MESSAGE_CREATE 相同
    # - MESSAGE_DELETE         // 删除（撤回）消息事件

    INTENT_MESSAGE_REACTION = 1 << 10
    # - MESSAGE_REACTION_ADD    // 为消息添加表情表态
    # - MESSAGE_REACTION_REMOVE // 为消息删除表情表态

    INTENT_DIRECT_MESSAGE = 1 << 12
    # - DIRECT_MESSAGE_CREATE // 当收到用户发给机器人的私信消息时
    # - DIRECT_MESSAGE_DELETE   // 删除（撤回）消息事件

    INTERACTION = 1 << 26
    # - INTERACTION_CREATE // 互动事件创建时

    INTENT_AUDIO = 1 << 29
    # - AUDIO_START           // 音频开始播放时
    # - AUDIO_FINISH          // 音频播放结束时
    # - AUDIO_ON_MIC          // 上麦时
    # - AUDIO_OFF_MIC         // 下麦时

    INTENT_PUBLIC_GUILD_MESSAGES = 1 << 30
    # - AT_MESSAGE_CREATE // 当收到 @ 机器人的消息时
    # - PUBLIC_MESSAGE_DELETE   // 当频道的消息被删除时



class Interaction:
    def __init__(self, data=None):
        self.id: str = ""
        self.application_id: int = 0
        self.type: int = 0
        self.data: InteractionData = None
        self.guild_id: int = 0
        self.channel_id: int = 0
        self.version: int = 1
        if data:
            self.__dict__ = data


class InteractionData:
    def __init__(
            self,
            type: int = 0,
            resolved: object = None,
            data=None
    ):
        self.type = type
        self.resolved = resolved
        if data:
            self.__dict__ = data


class InteractionType:
    PING = 1
    APPLICATION_COMMAND = 2
    HTTP_PROXY = 10
    INLINE_KEYBOARD = 11


class InteractionDataType:
    CHAT_INPUT_SEARCH = 9
    HTTP_PROXY = 10
    INLINE_KEYBOARD_BUTTON_CLICK = 11



# -*- coding: utf-8 -*-

import yaml


class YamlUtil:
    @staticmethod
    def read(yaml_path):
        """
        读取指定目录的yaml文件

        :param yaml_path: 相对当前的yaml文件绝对路径
        :return:
        """
        # 加上 ,encoding='utf-8'，处理配置文件中含中文出现乱码的情况。
        with open(yaml_path, "r", encoding="utf-8") as f:
            return yaml.safe_load(f)


if __name__ == "__main__":
    values = YamlUtil.read(".test.yaml")
    print(values)



# -*- coding: utf-8 -*-
import asyncio
import threading
import time

from qqbot.core.util import logging

logger = logging.getLogger()


class SessionPool:
    """
    SessionPool主要支持session的重连，可以根据session的状态动态设置是否需要进行重连操作
    这里通过设置session_id=""空则任务session需要重连
    """

    def __init__(self, max_async, session_manager, loop=None, session_count=1):
        self.max_async = max_async
        self.session_manager = session_manager
        self.loop = loop or asyncio.get_event_loop()
        self._queue = asyncio.Queue(maxsize=session_count, loop=self.loop)
        self.lock = threading.Lock()
        # session链接同时最大并发数
        self.work_list = []

    async def run(self, session_interval=5):
        works = [
            asyncio.ensure_future(self._work(session_interval), loop=self.loop)
            for _ in range(self.max_async)
        ]
        self.work_list.extend(works)
        await self._queue.join()
        logger.debug("all tasks done")

    async def _work(self, session_interval):

        try:
            while True:
                # 后台有限制，一定时间后发起链接
                time.sleep(session_interval)
                session = await self._queue.get()
                logger.debug("get session: %s" % session)
                # 这里开启线程添加websocket链接，不用等待
                thread = threading.Thread(
                    target=self.session_manager.new_connect, args=(session,)
                )
                thread.start()
                self._queue.task_done()
                if self._queue.empty():
                    break

        except asyncio.CancelledError:
            logger.error(asyncio.CancelledError)

    def add_task(self, item):
        with self.lock:
            self._queue.put_nowait(item)

    @property
    def count(self):
        return self._queue.qsize()



# -*- coding: utf-8 -*-
from .audio import AudioControl, STATUS
from .channel import Channel, ChannelType, ChannelSubType, PatchChannelRequest
from .channel import CreateChannelRequest
from .channel_permissions import UpdatePermission
from .guild import Guild
from .guild_member import GuildMember, QueryParams
from .guild_role import RoleUpdateInfo
from .message import (
    Message,
    TypesEnum,
    MessagesPager,
    MessageAttachment,
    MessageEmbedThumbnail,
    MessageEmbedField,
    MessageEmbed,
    MessageArkObjKv,
    MessageArkObj,
    MessageArkKv,
    MessageArk,
    MessageMarkdownParams,
    MessageMarkdown,
    MessageReference,
    MessageSendRequest,
    DirectMessageGuild,
    CreateDirectMessageRequest,
    DeletedMessageOriginalAuthor,
    DeletedMessage,
    DeletionOperator,
    DeletedMessageInfo,
)
from .mute import MuteOption, MultiMuteOption
from .schedule import ScheduleToCreate, ScheduleToPatch
from .token import Token
from .user import ReqOption
from .reaction import *
from .interaction import *



# -*- coding: utf-8 -*-
from typing import List


class MuteOption:
    def __init__(
            self,
            mute_end_timestamp: str = None,
            mute_seconds: str = None,
            ):
        """

        :param mute_end_timestamp: 禁言到期时间戳，绝对时间戳，单位：秒（与 seconds 字段同时赋值的话，以该字段为准）
        :param mute_seconds: 禁言多少秒（两个字段二选一，默认以 timeTo 为准）
        """
        if mute_end_timestamp is not None:
            self.mute_end_timestamp = mute_end_timestamp
        if mute_seconds is not None:
            self.mute_seconds = mute_seconds


class MultiMuteOption:
    def __init__(
            self,
            mute_end_timestamp: str = None,
            mute_seconds: str = None,
            user_ids: List[str] = None
            ):
        """

        :param mute_end_timestamp: 禁言到期时间戳，绝对时间戳，单位：秒（与 seconds 字段同时赋值的话，以该字段为准）
        :param mute_seconds: 禁言多少秒（两个字段二选一，默认以 timeTo 为准）
        """
        if mute_end_timestamp is not None:
            self.mute_end_timestamp = mute_end_timestamp
        if mute_seconds is not None:
            self.mute_seconds = mute_seconds
        if user_ids is not None:
            self.user_ids = user_ids


class UserIds:
    def __init__(self, data=None):
        self.user_ids: List[str]
        if data:
            self.__dict__ = data



from typing import List

from qqbot.model.member import User

from qqbot.model.emoji import Emoji


class Reaction:
    def __init__(self, data=None):
        self.user_id: str = ""
        self.guild_id: str = ""
        self.channel_id: str = ""
        self.target: ReactionTarget = ReactionTarget()
        self.emoji: Emoji = Emoji()
        if data:
            self.__dict__ = data


class ReactionTarget:
    def __init__(self, data=None):
        self.id: str = ""
        self.type: int = 0
        if data:
            self.__dict__ = data


class ReactionUsers:
    def __init__(
            self,
            data=None,
            users: List[User] = None,
            cookie: str = None,
            is_end: bool = None
    ):
        if users:
            self.users = users
        if cookie:
            self.cookie = cookie
        if is_end:
            self.is_end = is_end
        if data:
            self.__dict__ = data


class ReactionUsersPager:
    def __init__(self, cookie: str = "", limit: int = 20):
        self.cookie = cookie
        self.limit = limit


class ReactionTargetType:
    message = 0
    feed = 1
    comment = 2
    reply = 3



# -*- coding: utf-8 -*-
from typing import List


class Announce:
    def __init__(self, data=None):
        self.guild_id: str = ""
        self.channel_id: str = ""
        self.message_id: str = ""
        self.announces_type: int = 0
        self.recommend_channels: List[RecommendChannel] = []
        if data:
            self.__dict__ = data


class RecommendChannel:
    def __init__(self, channel_id: str, introduce: str):
        self.channel_id = channel_id
        self.introduce = introduce


class CreateAnnounceRequest:
    def __init__(self, channel_id: str, message_id: str):
        self.channel_id = channel_id
        self.message_id = message_id


class CreateChannelAnnounceRequest:
    def __init__(self, message_id: str):
        self.message_id = message_id


class RecommendChannelRequest:
    def __init__(self, announces_type: int, recommend_channels: List[RecommendChannel]):
        self.announces_type = announces_type
        self.recommend_channels = recommend_channels



# -*- coding: utf-8 -*-

import json

from qqbot.core.network.ws.ws_event import WsEvent
from qqbot.core.network.ws.ws_handler import DefaultHandler
from qqbot.core.util import logging
from qqbot.model.audio import AudioAction
from qqbot.model.channel import Channel
from qqbot.model.guild import Guild
from qqbot.model.guild_member import GuildMember
from qqbot.model.message import Message, DeletedMessageInfo
from qqbot.model.reaction import Reaction
from qqbot.model.interaction import Interaction
from qqbot.model.ws_context import WsContext

logger = logging.getLogger()


def parse_and_handle(ws_event, data):
    event_id = ws_event["id"] if "id" in ws_event.keys() else ""
    event_type = ws_event["t"]
    context = WsContext(event_type, event_id)
    callback = DefaultHandler.get_handler_by_type(event_type)

    if callback is None:
        if DefaultHandler.plain is not None:
            plain = DefaultHandler.plain
            plain(context, data)
        else:
            return
    else:
        object_type = event_object_dict.get(event_type)
        resolved = json.loads(_parse_data(data), object_hook=object_type)
        callback(context, resolved)


event_object_dict = {
    WsEvent.EventGuildCreate: Guild,
    WsEvent.EventGuildUpdate: Guild,
    WsEvent.EventGuildDelete: Guild,
    WsEvent.EventChannelCreate: Channel,
    WsEvent.EventChannelUpdate: Channel,
    WsEvent.EventChannelDelete: Channel,
    WsEvent.EventGuildMemberAdd: GuildMember,
    WsEvent.EventGuildMemberUpdate: GuildMember,
    WsEvent.EventGuildMemberRemove: GuildMember,
    WsEvent.EventAtMessageCreate: Message,
    WsEvent.EventPublicMessageDelete: DeletedMessageInfo,
    WsEvent.EventMessageCreate: Message,
    WsEvent.EventMessageDelete: DeletedMessageInfo,
    WsEvent.EventDirectMessageCreate: Message,
    WsEvent.EventDirectMessageDelete: DeletedMessageInfo,
    WsEvent.EventAudioStart: AudioAction,
    WsEvent.EventAudioFinish: AudioAction,
    WsEvent.EventAudioOnMic: AudioAction,
    WsEvent.EventAudioOffMic: AudioAction,
    WsEvent.EventMessageReactionAdd: Reaction,
    WsEvent.EventMessageReactionRemove: Reaction,
    WsEvent.EventInteractionCreate: Interaction,
}


def _parse_data(data_dict):
    json_obj = json.loads(data_dict)
    if "d" in json_obj.keys():
        data = json_obj["d"]
        return json.dumps(data)



# -*- coding: utf-8 -*-


class Type:
    typeBot = "Bot"
    typeNormal = "Bearer"


class Token:
    def __init__(self, app_id: str, access_token: str):
        """
        :param app_id: 机器人appid
        :param access_token: 机器人token
        """
        self.app_id = app_id
        self.access_token = access_token
        self.Type = Type.typeBot

    # BotToken 机器人身份的 token
    def bot_token(self):
        return self

    # GetString 获取授权头字符串
    def get_string(self):
        if self.Type == Type.typeNormal:
            return self.access_token
        return "{}.{}".format(self.app_id, self.access_token)

    def get_type(self):
        return self.Type



# -*- coding: utf-8 -*-
from typing import List


class PinsMessage:
    def __init__(self, data=None):
        self.guild_id: str = ""
        self.channel_id: str = ""
        self.message_ids: List[str] = []
        if data:
            self.__dict__ = data



from qqbot.core.network.ws.ws_event import WsEvent


class DefaultHandler:
    """
    持有handler的实例
    """

    plain = None
    guild = None
    guild_member = None
    channel = None
    message_create = None
    message_delete = None
    at_message = None
    public_message_delete = None
    direct_message_create = None
    direct_message_delete = None
    audio = None
    message_reaction = None
    interaction_create = None

    @classmethod
    def get_handler_by_type(cls, event_type: str):
        if event_type in (
                WsEvent.EventGuildCreate,
                WsEvent.EventGuildUpdate,
                WsEvent.EventGuildDelete
        ):
            return cls.guild
        elif event_type in (
                WsEvent.EventChannelCreate,
                WsEvent.EventChannelUpdate,
                WsEvent.EventChannelDelete
        ):
            return cls.channel
        elif event_type in (
            WsEvent.EventGuildMemberAdd,
            WsEvent.EventGuildMemberUpdate,
            WsEvent.EventGuildMemberRemove,
        ):
            return cls.guild_member
        elif event_type == WsEvent.EventAtMessageCreate:
            return cls.at_message
        elif event_type == WsEvent.EventPublicMessageDelete:
            return cls.public_message_delete
        elif event_type == WsEvent.EventMessageCreate:
            return cls.message_create
        elif event_type == WsEvent.EventMessageDelete:
            return cls.message_delete
        elif event_type == WsEvent.EventDirectMessageCreate:
            return cls.direct_message_create
        elif event_type == WsEvent.EventDirectMessageDelete:
            return cls.direct_message_delete
        elif event_type in (
            WsEvent.EventAudioStart,
            WsEvent.EventAudioFinish,
            WsEvent.EventAudioOnMic,
            WsEvent.EventAudioOffMic,
        ):
            return cls.audio
        elif event_type in (
                WsEvent.EventMessageReactionAdd,
                WsEvent.EventMessageReactionRemove
        ):
            return cls.message_reaction
        elif event_type == WsEvent.EventInteractionCreate:
            return cls.interaction_create



# -*- coding: utf-8 -*-
import asyncio

from qqbot.core.util import logging

logger = logging.getLogger()


class SessionPool:
    """
    SessionPool主要支持session的重连，可以根据session的状态动态设置是否需要进行重连操作
    这里通过设置session_id=""空则任务session需要重连
    """

    def __init__(self, max_async, session_manager, loop=None):
        self.max_async = max_async
        self.session_manager = session_manager
        self.loop: asyncio.AbstractEventLoop = asyncio.get_event_loop() if loop is None else loop
        # session链接同时最大并发数
        self.session_list = []

    async def run(self, session_interval=5):
        loop = self.loop

        # 根据并发数同时建立多个future
        index = 0
        session_list = self.session_list
        # 需要执行的链接列表，通过time_interval控制启动时间
        tasks = []

        while len(session_list) > 0:
            logger.debug("session list circle run")
            time_interval = session_interval * (index + 1)
            logger.info("[连接池]最大并发连接数: %s, 启动会话数: %s" % (self.max_async, len(session_list)))
            for i in range(self.max_async):
                if len(session_list) == 0:
                    break
                tasks.append(asyncio.ensure_future(self._runner(session_list.pop(i), time_interval), loop=loop))
            index += self.max_async

        await asyncio.wait(tasks)

    async def _runner(self, session, time_interval):
        await self.session_manager.new_connect(session)
        # 后台有频率限制，根据间隔时间发起链接请求
        await asyncio.sleep(time_interval)

    def add(self, session):
        self.session_list.append(session)

    def close(self):
        self.loop.close()
